@@ -26,7 +26,8 @@ gmc_execute (char *fname, char *buf)
 int
 gmc_open_filename (char *fname, GList *args)
 {
-	char *mime_type, *cmd;
+	const char *mime_type;
+	const char *cmd;
 	char *buf;
 	int size;
 
@@ -59,9 +60,51 @@ gmc_open_filename (char *fname, GList *args)
 		gmc_execute (fname, cmd);
 		return 1;
 	}
+
+	if (strcmp (mime_type, "application/x-gnome-app-info")  == 0){
+		GnomeDesktopEntry *entry;
+
+		entry = gnome_desktop_entry_load (fname);
+		if (entry){
+			gnome_desktop_entry_launch (entry);
+			gnome_desktop_entry_free (entry);
+		}
+	}
+	
 	return 0;
 }
 
+int
+gmc_edit_filename (char *fname)
+{
+	const char *mime_type;
+	const char *cmd;
+	char *buf;
+	int size;
+
+	if (gnome_metadata_get (fname, "edit", &size, &buf) == 0){
+		gmc_execute (fname, buf);
+		g_free (buf);
+		return 1;
+	}
+
+	mime_type = gnome_mime_type_or_default (fname, NULL);
+	if (!mime_type)
+		return 0;
+	
+		
+	cmd = gnome_mime_get_value (mime_type, "edit");
+	
+	if (cmd){
+		gmc_execute (fname, cmd);
+		return 1;
+	}
+	
+	do_edit (fname);
+	return 0;
+}
+
+
 int
 gmc_open (file_entry *fe)
 {
@@ -77,7 +120,7 @@ gmc_run_view (char *filename, char *buf)
 int
 gmc_view (char *filename, int start_line)
 {
-	char *mime_type, *cmd;
+	const char *mime_type, *cmd;
 	char *buf;
 	int size;
 
