@@ -7,7 +7,7 @@ void gnome_open_terminal     (void);
 void gnome_about_cmd         (void);
 void gnome_quit_cmd          (void);
 void gnome_open_panel        (GtkWidget *widget, WPanel *panel);
-int  gnome_close_panel       (GtkWidget *widget, WPanel *panel);
+void gnome_close_panel       (GtkWidget *widget, WPanel *panel);
 void gnome_icon_view_cmd     (GtkWidget *widget, WPanel *panel);
 void gnome_partial_view_cmd  (GtkWidget *widget, WPanel *panel);
 void gnome_full_view_cmd     (GtkWidget *widget, WPanel *panel);
