@@ -1,13 +1,13 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /* New dialogs... */
 #include <config.h>
-#include "util.h"
 #include "panel.h"
 #include <gnome.h>
+#include "global.h"
 #include "file.h"
 #include "filegui.h"
 #include "fileopctx.h"
-#include "regex.h"
+#include "eregex.h"
 #include "../vfs/vfs.h"
 
 enum {
@@ -55,13 +55,12 @@ typedef struct {
         GtkWidget *op_radio;
 } FileOpContextUI;
 
+static char *gdialog_to_string = N_("To: ");
+static char *gdialog_from_string = N_("Copying from: ");
+static char *gdialog_deleting_string = N_("Deleting file: ");
 
-#define GDIALOG_TO_STRING "To: "
-#define GDIALOG_FROM_STRING "Copying from: "
 #define GDIALOG_PROGRESS_WIDTH 350
 
-
-
 /* Callbacks go here... */
 static void
 fmd_check_box_callback (GtkWidget *widget, gpointer data)
@@ -93,8 +92,7 @@ trim_file_name (FileOpContextUI *ui, gchar *path, gint length, gint cur_length)
                 while (gdk_string_width (ui->op_source_label->style->font, path + len) > length)
                         len ++;
         }
-        path_copy = g_malloc (sizeof (gchar [1 + 3 + strlen (path) - len]));
-        sprintf (path_copy, "...%s", path + len);
+        path_copy = g_strdup_printf ("...%s", path + len);
         return path_copy;
 }
 
@@ -117,22 +115,22 @@ file_progress_show_source (FileOpContext *ctx, char *path)
                 return FILE_ABORT;
 
         if (path == NULL){
-                gtk_label_set (GTK_LABEL (ui->op_source_label), "");
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), "");
                 return FILE_CONT;
         }
 
         if (!from_width){
                 from_width = gdk_string_width (ui->op_source_label->style->font,
-                                               _(GDIALOG_FROM_STRING));
+                                               _(gdialog_from_string));
         }
         path_width = gdk_string_width (ui->op_source_label->style->font, path);
         if (from_width + path_width < GDIALOG_PROGRESS_WIDTH)
-                gtk_label_set (GTK_LABEL (ui->op_source_label), path);
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), path);
         else {
                 path_copy = trim_file_name (ui, path, GDIALOG_PROGRESS_WIDTH - from_width,
                                             path_width);
                 
-                gtk_label_set (GTK_LABEL (ui->op_source_label), path_copy);
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), path_copy);
                 g_free (path_copy);
         }
 
@@ -158,19 +156,19 @@ file_progress_show_target (FileOpContext *ctx, char *path)
                 return FILE_ABORT;
 
         if (path == NULL){
-                gtk_label_set (GTK_LABEL (ui->op_target_label), "");
+                gtk_label_set_text (GTK_LABEL (ui->op_target_label), "");
                 return FILE_CONT;
         }
 
         if (!to_width) 
                 to_width = gdk_string_width (ui->op_target_label->style->font,
-                                             _(GDIALOG_TO_STRING));
+                                             _(gdialog_to_string));
         path_width = gdk_string_width (ui->op_target_label->style->font, path);
         if (to_width + path_width < GDIALOG_PROGRESS_WIDTH)
-                gtk_label_set (GTK_LABEL (ui->op_target_label), path);
+                gtk_label_set_text (GTK_LABEL (ui->op_target_label), path);
         else {
                 path_copy = trim_file_name (ui, path, GDIALOG_PROGRESS_WIDTH - to_width, path_width);
-                gtk_label_set (GTK_LABEL (ui->op_target_label), path_copy);
+                gtk_label_set_text (GTK_LABEL (ui->op_target_label), path_copy);
                 g_free (path_copy);
         }
 
@@ -180,7 +178,10 @@ file_progress_show_target (FileOpContext *ctx, char *path)
 FileProgressStatus
 file_progress_show_deleting (FileOpContext *ctx, char *path)
 {
+        static gint deleting_width = 0;
         FileOpContextUI *ui;
+        gint path_width;
+        gchar *path_copy = NULL;
 
         g_return_val_if_fail (ctx != NULL, FILE_CONT);
         g_return_val_if_fail (ctx->ui != NULL, FILE_CONT);
@@ -190,8 +191,25 @@ file_progress_show_deleting (FileOpContext *ctx, char *path)
         if (ui->aborting)
                 return FILE_ABORT;
 
-        /* FIXME */
-	g_warning ("memo: file_progress_show_deleting!\npath\t%s\n", path);
+        if (path == NULL){
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), "");
+                return FILE_CONT;
+        }
+
+        if (!deleting_width){
+                deleting_width = gdk_string_width (ui->op_source_label->style->font,
+                                                   _(gdialog_deleting_string));
+        }
+        path_width = gdk_string_width (ui->op_source_label->style->font, path);
+        if (deleting_width + path_width < GDIALOG_PROGRESS_WIDTH)
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), path);
+        else {
+                path_copy = trim_file_name (ui, path, GDIALOG_PROGRESS_WIDTH - deleting_width,
+                                            path_width);
+                
+                gtk_label_set_text (GTK_LABEL (ui->op_source_label), path_copy);
+                g_free (path_copy);
+        }
         return FILE_CONT;
 }
 
@@ -210,7 +228,7 @@ file_progress_show (FileOpContext *ctx, long done, long total)
                 return FILE_ABORT;
         
         snprintf (count, 9, "%d%%", (gint)(100.0 *(gfloat)done/(gfloat)total));
-        gtk_label_set (GTK_LABEL (ui->file_label), count);
+        gtk_label_set_text (GTK_LABEL (ui->file_label), count);
         while (gtk_events_pending ())
                 gtk_main_iteration ();
 	return FILE_CONT;
@@ -231,7 +249,7 @@ file_progress_show_count (FileOpContext *ctx, long done, long total)
                 return FILE_ABORT;
 
         snprintf (count, 13, "%d/%d", done, total);
-        gtk_label_set (GTK_LABEL (ui->count_label), count);
+        gtk_label_set_text (GTK_LABEL (ui->count_label), count);
         while (gtk_events_pending ())
                 gtk_main_iteration ();
 	return FILE_CONT;
@@ -284,7 +302,6 @@ policy_callback (GtkWidget *button, gpointer data)
         status = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (button)));
 
         if (GTK_TOGGLE_BUTTON (button)->active) {
-                g_print ("toggle\n");
                 if (status == REPLACE_OPTION_MENU) {
                         ui->copy_status = ui->minor_copy_status;
                 } else
@@ -320,11 +337,11 @@ file_progress_query_replace_policy (FileOpContext *ctx, gboolean dialog_needed)
         if (dialog_needed == FALSE)
                 return FILE_CONT;
         ui->minor_copy_status = REPLACE_ALWAYS;
-        g_print ("in file_progress_query_replace_policy\n");
         qrp_dlg = gnome_dialog_new (_("Files Exist"),
                                     GNOME_STOCK_BUTTON_OK,
                                     GNOME_STOCK_BUTTON_CANCEL,
                                     NULL);
+        gtk_window_set_position (GTK_WINDOW (qrp_dlg), GTK_WIN_POS_MOUSE);
         vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
         hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
         icon = gnome_stock_pixmap_widget (hbox, GNOME_STOCK_PIXMAP_HELP);
@@ -433,6 +450,7 @@ file_progress_real_query_replace (FileOpContext *ctx, enum OperationMode mode, c
                                            GNOME_STOCK_BUTTON_NO,
                                            GNOME_STOCK_BUTTON_CANCEL, NULL);
 
+                gtk_window_set_position (GTK_WINDOW (qr_dlg), GTK_WIN_POS_MOUSE);
                 snprintf (msg, sizeof (msg)-1, _("The target file already exists: %s"), destname);
                 label = gtk_label_new (msg);
                 gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
@@ -517,6 +535,8 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text, char
                 fmd_win = gnome_dialog_new (_("Move"), GNOME_STOCK_BUTTON_OK, 
                                             GNOME_STOCK_BUTTON_CANCEL, NULL);
 
+        gtk_window_set_position (GTK_WINDOW (fmd_win), GTK_WIN_POS_MOUSE);
+        
         hbox = gtk_hbox_new (FALSE, GNOME_PAD);
         notebook = gtk_notebook_new ();
         gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (fmd_win)->vbox),
@@ -631,11 +651,11 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text, char
                 umask (i);
                 ctx->umask_kill = i ^ 0777777;
         }
-        source_mask = strdup ("*");
+        source_mask = g_strdup ("*");
         orig_mask = source_mask;
         if (!dest_dir || !*dest_dir){
                 if (source_mask)
-                        free (source_mask);
+                        g_free (source_mask);
                 return dest_dir;
         }
 
@@ -653,7 +673,7 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text, char
                 source_mask = convert_pattern (source_mask, match_file, 1);
                 easy_patterns = source_easy_patterns;
                 error = re_compile_pattern (source_mask, strlen (source_mask), &ctx->rx);
-                free (source_mask);
+                g_free (source_mask);
         } else
                 error = re_compile_pattern (source_mask, strlen (source_mask), &ctx->rx);
 
@@ -661,7 +681,7 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text, char
                 g_warning ("%s\n",error);
 
         if (orig_mask)
-                free (orig_mask);
+                g_free (orig_mask);
         ctx->dest_mask = strrchr (dest_dir, PATH_SEP);
         if (ctx->dest_mask == NULL)
                 ctx->dest_mask = dest_dir;
@@ -674,14 +694,14 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text, char
             (ctx->dive_into_subdirs && ((!only_one && !is_wildcarded (ctx->dest_mask)) ||
                                         (only_one && !mc_stat (dest_dir, &buf)
                                          && S_ISDIR (buf.st_mode)))))
-                ctx->dest_mask = strdup ("*");
+                ctx->dest_mask = g_strdup ("*");
         else {
-                ctx->dest_mask = strdup (ctx->dest_mask);
+                ctx->dest_mask = g_strdup (ctx->dest_mask);
                 *orig_mask = 0;
         }
         if (!*dest_dir){
-                free (dest_dir);
-                dest_dir = strdup ("./");
+                g_free (dest_dir);
+                dest_dir = g_strdup ("./");
         }
         return dest_dir;
 }
@@ -710,42 +730,55 @@ file_op_context_create_ui (FileOpContext *ctx, FileOperation op, int with_eta)
 
         switch (op) {
         case OP_MOVE:
-                ui->op_win = gnome_dialog_new ("Move Progress", GNOME_STOCK_BUTTON_CANCEL, NULL);
+                ui->op_win = gnome_dialog_new (_("Move Progress"), GNOME_STOCK_BUTTON_CANCEL, NULL);
                 break;
         case OP_COPY:
-                ui->op_win = gnome_dialog_new ("Copy Progress", GNOME_STOCK_BUTTON_CANCEL, NULL);
+                ui->op_win = gnome_dialog_new (_("Copy Progress"), GNOME_STOCK_BUTTON_CANCEL, NULL);
                 break;
         case OP_DELETE:
-                ui->op_win = gnome_dialog_new ("Delete Progress", GNOME_STOCK_BUTTON_CANCEL, NULL);
-                gtk_widget_show_all (GNOME_DIALOG (ui->op_win)->vbox);
-                gtk_widget_show_now (ui->op_win);
-                return;
+                ui->op_win = gnome_dialog_new (_("Delete Progress"), GNOME_STOCK_BUTTON_CANCEL, NULL);
+                break;
         }
+        gtk_window_set_position (GTK_WINDOW (ui->op_win), GTK_WIN_POS_MOUSE);
 
         gnome_dialog_button_connect (GNOME_DIALOG (ui->op_win), 0,
                                      GTK_SIGNAL_FUNC (cancel_cb), ui);
 
-        alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
-        hbox = gtk_hbox_new (FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (alignment), hbox);
-        gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(GDIALOG_FROM_STRING)), FALSE, FALSE, 0);
-        ui->op_source_label = gtk_label_new ("");
+        if (op != OP_DELETE) {
+                alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
+                hbox = gtk_hbox_new (FALSE, 0);
+                gtk_container_add (GTK_CONTAINER (alignment), hbox);
+                gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(gdialog_from_string)), FALSE, FALSE, 0);
+                ui->op_source_label = gtk_label_new ("");
 
-        gtk_box_pack_start (GTK_BOX (hbox), ui->op_source_label, FALSE, FALSE, 0);
-        gtk_box_set_spacing (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD_SMALL);
-        gtk_container_set_border_width (GTK_CONTAINER (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD);
-        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
-                            alignment, FALSE, FALSE, 0);
+                gtk_box_pack_start (GTK_BOX (hbox), ui->op_source_label, FALSE, FALSE, 0);
+                gtk_box_set_spacing (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD_SMALL);
+                gtk_container_set_border_width (GTK_CONTAINER (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD);
+                gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
+                                    alignment, FALSE, FALSE, 0);
 
-        alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
-        hbox = gtk_hbox_new (FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (alignment), hbox);
-        gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(GDIALOG_TO_STRING)), FALSE, FALSE, 0);
-        ui->op_target_label = gtk_label_new ("");
-        gtk_box_pack_start (GTK_BOX (hbox), ui->op_target_label, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
-                            alignment, FALSE, FALSE, 0);
+                alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
+                hbox = gtk_hbox_new (FALSE, 0);
+                gtk_container_add (GTK_CONTAINER (alignment), hbox);
+                gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(gdialog_to_string)), FALSE, FALSE, 0);
+                ui->op_target_label = gtk_label_new ("");
+                gtk_box_pack_start (GTK_BOX (hbox), ui->op_target_label, FALSE, FALSE, 0);
+                gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
+                                    alignment, FALSE, FALSE, 0);
 
+        } else {
+                alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
+                hbox = gtk_hbox_new (FALSE, 0);
+                gtk_container_add (GTK_CONTAINER (alignment), hbox);
+                gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(gdialog_deleting_string)), FALSE, FALSE, 0);
+                ui->op_source_label = gtk_label_new ("");
+                
+                gtk_box_pack_start (GTK_BOX (hbox), ui->op_source_label, FALSE, FALSE, 0);
+                gtk_box_set_spacing (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD_SMALL);
+                gtk_container_set_border_width (GTK_CONTAINER (GNOME_DIALOG (ui->op_win)->vbox), GNOME_PAD);
+                gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
+                                    alignment, FALSE, FALSE, 0);
+        }        
         alignment = gtk_alignment_new (0.0, 0.5, 0, 0);
         hbox = gtk_hbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("File ")), FALSE, FALSE, 0);
@@ -756,7 +789,6 @@ file_op_context_create_ui (FileOpContext *ctx, FileOperation op, int with_eta)
         ui->file_label = gtk_label_new ("");
         gtk_box_pack_start (GTK_BOX (hbox), ui->file_label, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_(" Done.")), FALSE, FALSE, 0);
-
         gtk_container_add (GTK_CONTAINER (alignment), hbox);
 
         gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ui->op_win)->vbox),
