@@ -19,6 +19,7 @@
 
 #include <gnome.h>
 #include <time.h>
+#include <string.h>
 #include "gnome-file-property-dialog.h"
 #include "dir.h"
 #include "gdesktop.h"
@@ -31,6 +32,7 @@
 #include "../vfs/vfs.h"
 #include "gicon.h"
 #include "dialog.h"
+#include "util.h"
 
 static void gnome_file_property_dialog_init		(GnomeFilePropertyDialog	 *file_property_dialog);
 static void gnome_file_property_dialog_class_init	(GnomeFilePropertyDialogClass	 *klass);
@@ -110,6 +112,21 @@ gnome_file_property_dialog_finalize (GtkObject *object)
 		g_free (gfpd->file_name);
 	if (gfpd->im)
 		gdk_imlib_destroy_image (gfpd->im);
+	if (gfpd->fm_open)
+		g_free (gfpd->fm_open);
+	if (gfpd->fm_view)
+		g_free (gfpd->fm_view);
+	if (gfpd->edit)
+		g_free (gfpd->edit);
+	if (gfpd->drop_target)
+		g_free (gfpd->drop_target);
+	if (gfpd->icon_filename)
+		g_free (gfpd->icon_filename);
+	if (gfpd->desktop_url)
+		g_free (gfpd->desktop_url);
+	if (gfpd->caption)
+		g_free (gfpd->caption);
+	
 	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -140,6 +157,7 @@ create_general_properties (GnomeFilePropertyDialog *fp_dlg)
 	strrchr (direc, '/')[0] = '\0';
 	fe = file_entry_from_file (fp_dlg->file_name);
 	fp_dlg->im = gicon_get_icon_for_file_speed (direc, fe, FALSE);
+	file_entry_free (fe);
 	g_free (direc);
 	icon = gnome_pixmap_new_from_imlib (fp_dlg->im);
 	gtk_box_pack_start (GTK_BOX (vbox), icon, FALSE, FALSE, 0);
@@ -156,7 +174,7 @@ create_general_properties (GnomeFilePropertyDialog *fp_dlg)
 	label = gtk_label_new (_("File Name"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	fp_dlg->file_entry = gtk_entry_new ();
-	gen_string = rindex (fp_dlg->file_name, '/');
+	gen_string = strrchr (fp_dlg->file_name, '/');
 	if (gen_string)
 		gtk_entry_set_text (GTK_ENTRY (fp_dlg->file_entry), gen_string + 1);
 	else
@@ -270,7 +288,30 @@ create_general_properties (GnomeFilePropertyDialog *fp_dlg)
 	return vbox;
 }
 
+static GtkWidget *
+create_url_properties (GnomeFilePropertyDialog *fp_dlg)
+{
+	GtkWidget *table, *label;
 
+	table = gtk_table_new (0, 0, 0);
+
+	label = gtk_label_new (_("URL:"));
+	gtk_table_attach (GTK_TABLE (table), label,
+			  1, 2, 1, 2, 0, 0, GNOME_PAD, GNOME_PAD);
+	fp_dlg->desktop_entry = gtk_entry_new ();
+	gtk_table_attach (GTK_TABLE (table), fp_dlg->desktop_entry,
+			  2, 3, 1, 2, GTK_FILL | GTK_EXPAND, 0, GNOME_PAD, GNOME_PAD);
+
+	label = gtk_label_new (_("Caption:"));
+	gtk_table_attach (GTK_TABLE (table), label,
+			  1, 2, 2, 3, 0, 0, GNOME_PAD, GNOME_PAD);
+	fp_dlg->caption_entry = gtk_entry_new ();
+	gtk_table_attach (GTK_TABLE (table), fp_dlg->caption_entry,
+			  2, 3, 2, 3, GTK_FILL | GTK_EXPAND, 0, GNOME_PAD, GNOME_PAD);
+	
+	gtk_widget_show_all (table);
+	return table;
+}
 
 /* Settings Pane */
 static void
@@ -329,6 +370,13 @@ switch_metadata_box (GnomeFilePropertyDialog *fp_dlg)
 	if (NULL == fp_dlg->prop1_label)
 		return;
 	fp_dlg->changing = TRUE;
+
+	if (fp_dlg->desktop_url){
+		gtk_entry_set_text (GTK_ENTRY (fp_dlg->desktop_entry), fp_dlg->desktop_url);
+
+		gtk_entry_set_text (GTK_ENTRY (fp_dlg->caption_entry), fp_dlg->caption);
+	}
+
 	if (fp_dlg->executable) {
 		gtk_label_set_text (GTK_LABEL (fp_dlg->prop1_label), "Drop Action");
 		gtk_label_set_text (GTK_LABEL (GTK_BIN (fp_dlg->prop1_cbox)->child), "Use default Drop Action options");
@@ -383,10 +431,15 @@ generate_icon_sel (GnomeFilePropertyDialog *fp_dlg)
 	
 	retval = gnome_icon_entry_new ("gmc_file_icon", "Select an Icon");
 	icon = gicon_image_to_name (fp_dlg->im);
-	if (!icon || !icon[0])
+	if (!icon)
 		return retval;
-	g_print ("icon:%s:\n",icon);
+	
+	if (!icon[0]){
+		g_free (icon);
+		return retval;
+	}
 	gnome_icon_entry_set_icon (GNOME_ICON_ENTRY (retval), icon);
+	g_free (icon);
 	return retval;
 }
 static GtkWidget *
@@ -761,7 +814,7 @@ perm_group_new (GnomeFilePropertyDialog *fp_dlg)
 		if (grp->gr_name)
 			fp_dlg->group_name = g_strdup (grp->gr_name);
 		else {
-			g_snprintf (grpnum, 9, "%d", grp->gr_gid);
+			g_snprintf (grpnum, 9, "%d", (int) grp->gr_gid);
 			fp_dlg->group_name = g_strdup (grpnum);
 		}
 
@@ -771,7 +824,7 @@ perm_group_new (GnomeFilePropertyDialog *fp_dlg)
 			if (grp->gr_name)
 				grpname = grp->gr_name;
 			else {
-				g_snprintf (grpnum, 9, "%d", grp->gr_gid);
+				g_snprintf (grpnum, 9, "%d", (int) grp->gr_gid);
 				grpname = grpnum;
 			}
 		}
@@ -884,10 +937,10 @@ static void
 init_metadata (GnomeFilePropertyDialog *fp_dlg)
 {
 	gint size;
-	gchar *mime_type;
+	char *mime_type;
 	gchar link_name[60];
 	gint n;
-	gchar *file_name;
+	gchar *file_name, *desktop_url;
 
 	if (gnome_metadata_get (fp_dlg->file_name, "fm-open", &size, &fp_dlg->fm_open) != 0)
 		gnome_metadata_get (fp_dlg->file_name, "open", &size, &fp_dlg->fm_open);
@@ -906,7 +959,18 @@ init_metadata (GnomeFilePropertyDialog *fp_dlg)
 		}
 	}
 	
-	mime_type = gnome_mime_type_or_default (file_name, NULL);
+	if (gnome_metadata_get (fp_dlg->file_name, "desktop-url", &size, &desktop_url) == 0)
+		fp_dlg->desktop_url = desktop_url;
+	else
+		fp_dlg->desktop_url = NULL;
+
+	if (gnome_metadata_get (fp_dlg->file_name, "icon-caption", &size, &fp_dlg->caption))
+		fp_dlg->caption = g_strdup (desktop_url);
+			
+	/*
+	 * Mime type.
+	 */
+	mime_type = (char *) gnome_mime_type_or_default (file_name, NULL);
 	if (!mime_type)
 		return;
 	fp_dlg->mime_fm_open = gnome_mime_get_value (mime_type, "fm-open");
@@ -921,6 +985,7 @@ init_metadata (GnomeFilePropertyDialog *fp_dlg)
 	gnome_metadata_get (fp_dlg->file_name, "icon-filename", &size, &fp_dlg->icon_filename);
 	if (fp_dlg->icon_filename)
 		g_print ("we have an icon-filename:%s:\n", fp_dlg->icon_filename);
+
 }
 GtkWidget *
 gnome_file_property_dialog_new (gchar *file_name, gboolean can_set_icon)
@@ -957,9 +1022,15 @@ gnome_file_property_dialog_new (gchar *file_name, gboolean can_set_icon)
 
 	/* and now, we set up the gui. */
 	notebook = gtk_notebook_new ();
+
+	if (fp_dlg->desktop_url)
+		gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+					  create_url_properties (fp_dlg),
+					  gtk_label_new (_("URL")));
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
 				  create_general_properties (fp_dlg),
 				  gtk_label_new (_("Statistics")));
+	
 	new_page = create_settings_pane (fp_dlg);
 	if (new_page)
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -970,9 +1041,10 @@ gnome_file_property_dialog_new (gchar *file_name, gboolean can_set_icon)
 				  gtk_label_new (_("Permissions")));
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (fp_dlg)->vbox),
 			    notebook, TRUE, TRUE, 0);
-	title_string = g_strconcat (rindex (file_name, '/') + 1, _(" Properties"), NULL);
+	title_string = g_strconcat (strrchr (file_name, '/') + 1, _(" Properties"), NULL);
 	gtk_window_set_title (GTK_WINDOW (fp_dlg), title_string);
 	g_free (title_string);
+	
 	gnome_dialog_append_button ( GNOME_DIALOG(fp_dlg), 
 				     GNOME_STOCK_BUTTON_OK);
 	gnome_dialog_append_button ( GNOME_DIALOG(fp_dlg), 
@@ -1053,9 +1125,9 @@ apply_uid_group_change (GnomeFilePropertyDialog *fpd)
 static gint
 apply_name_change (GnomeFilePropertyDialog *fpd)
 {
-	gchar *new_name;
-	gchar *base_name;
-	gchar *full_target;
+	char *new_name;
+	char *base_name;
+	char *full_target;
 	FileOpContext *ctx;
 	long   count = 0;
 	double bytes = 0;
@@ -1066,14 +1138,21 @@ apply_name_change (GnomeFilePropertyDialog *fpd)
 		return 0;
 	}
 	/* has it changed? */
-	if (strcmp (rindex(fpd->file_name, '/') + 1, new_name)) {
+	if (strcmp (strrchr(fpd->file_name, '/') + 1, new_name)) {
 		if (strchr (new_name, '/')) {
 			message (1, "Error", _("You cannot rename a file to something containing a '/' character"));
 			return 0;
 		} else {
+			char *p;
+			
 			/* create the files. */
 			base_name = g_strdup (fpd->file_name);
-			rindex (base_name, '/')[0] = '\0';
+			
+			p = strrchr (base_name, '/');
+
+			if (p)
+				*p = '\0';
+			
 			full_target = concat_dir_and_file (base_name, new_name);
 
 			ctx = file_op_context_new ();
@@ -1159,6 +1238,17 @@ apply_metadata_change (GnomeFilePropertyDialog *fpd)
 			}
 		}
 	}
+
+	if (fpd->desktop_url){
+		char *new_desktop_url = gtk_entry_get_text (GTK_ENTRY (fpd->desktop_entry));
+		char *new_caption     = gtk_entry_get_text (GTK_ENTRY (fpd->caption_entry));
+
+		gnome_metadata_set (fpd->file_name, "desktop-url",
+				    strlen (new_desktop_url)+1, new_desktop_url);
+		gnome_metadata_set (fpd->file_name, "icon-caption",
+				    strlen (new_caption)+1, new_caption);
+	}
+		
 	if (!fpd->can_set_icon)
 		return 1;
 	/* And finally, we set the metadata on the icon filename */
@@ -1170,8 +1260,14 @@ apply_metadata_change (GnomeFilePropertyDialog *fpd)
 			 * default to the basic icon.  We prolly should check that this is a valid
 			 * file here, but I'm too tired to do it now -- jrb */
 			gnome_metadata_set (fpd->file_name, "icon-filename", strlen (text) + 1, text);
-		else
-			gnome_metadata_remove (fpd->file_name, "icon-filename");
+		else {
+			/* If text is equal to icon_name it means the user did not
+			 * touch it, not that he did remove it
+			 */
+			/*
+			  gnome_metadata_remove (fpd->file_name, "icon-filename");
+			*/
+		}
 	}
 	/* I suppose we should only do this if we know there's been a change -- I'll try to figure it
 	 * out later if it turns out to be important. */
