@@ -10,14 +10,12 @@
 
 #include <config.h>
 #include <string.h>
-#include <stdlib.h>		/* atoi */
 #include <pwd.h>
 #include <grp.h>
 #include <sys/types.h>
-#include "fs.h"
 #include "x.h"
+#include "global.h"
 #include "gprop.h"
-#include "util.h"
 #include "dialog.h"
 #include "file.h"
 #include "fileopctx.h"
@@ -68,8 +66,6 @@ item_properties (GtkWidget *parent, char *fname, DesktopIconInfo *dii)
 	char          *new_name;
 	char          *base;
 	char          *icon_filename;
-	int            size;
-		
 
 	struct stat s;
 	int retval = 0;
@@ -96,10 +92,16 @@ item_properties (GtkWidget *parent, char *fname, DesktopIconInfo *dii)
 	if (dii) {
 		GdkImlibImage *icon;
 		file_entry *fe;
-		char *name;
-		
+		char *dirname;
+		char *p;
+
+		p = strrchr (fname, PATH_SEP);
+		g_assert (p != NULL);
+		dirname = g_strndup (fname, p - fname);
+
 		fe = file_entry_from_file (fname);
-		icon = gicon_get_icon_for_file_speed (fe, FALSE);
+		icon = gicon_get_icon_for_file_speed (dirname, fe, FALSE);
+		g_free (dirname);
 		file_entry_free (fe);
 		icon_filename = gicon_image_to_name (icon);
 		if (icon_filename == NULL)
@@ -227,10 +229,10 @@ item_properties (GtkWidget *parent, char *fname, DesktopIconInfo *dii)
 		file_op_context_destroy (ctx);
 			
 		if (dii) {
-			free (dii->filename);
+			 g_free (dii->filename);
 			dii->filename = full_target;
 		} else
-			free (full_target);
+			 g_free (full_target);
 			
 		retval |= GPROP_FILENAME;
 	}
