@@ -78,12 +78,6 @@ GtkWidget *drag_multiple     = NULL;
 GtkWidget *drag_multiple_ok  = NULL;
 
 
-#define CLIST_FROM_SW(panel_list) GTK_CLIST (GTK_BIN (panel_list)->child)
-#ifdef NEW_GNOME_LIBS
-#define ILIST_FROM_SW(panel_list) GNOME_ICON_LIST (GTK_BIN (panel_list)->child)
-#endif
-
-
 void
 repaint_file (WPanel *panel, int file_index, int move, int attr, int isstatus)
 {
@@ -223,11 +217,7 @@ panel_fill_panel_list (WPanel *panel)
 static void
 panel_fill_panel_icons (WPanel *panel)
 {
-#ifdef NEW_GNOME_LIBS
 	GnomeIconList *icons = ILIST_FROM_SW (panel->icons);
-#else
-	GnomeIconList *icons = GNOME_ICON_LIST (panel->icons);
-#endif
 	const int top       = panel->count;
 	const int selected  = panel->selected;
 	int i;
@@ -320,11 +310,7 @@ x_select_item (WPanel *panel)
 	display_mini_info (panel);
 	
 	if (panel->list_type == list_icons){
-#ifdef NEW_GNOME_LIBS
 		GnomeIconList *list = ILIST_FROM_SW (panel->icons);
-#else
-		GnomeIconList *list = GNOME_ICON_LIST (panel->icons);
-#endif
 
 		gnome_icon_list_select_icon (list, panel->selected);
 
@@ -351,11 +337,7 @@ x_unselect_item (WPanel *panel)
 	int selected = panel->selected;
 	
 	if (panel->list_type == list_icons)
-#ifdef NEW_GNOME_LIBS
 		gnome_icon_list_unselect_all (ILIST_FROM_SW (panel->icons), NULL, NULL);
-#else
-		gnome_icon_list_unselect_all (GNOME_ICON_LIST (panel->icons), NULL, NULL);
-#endif
 	else
 		gtk_clist_unselect_all (CLIST_FROM_SW (panel->list));
 
@@ -484,7 +466,8 @@ panel_file_list_press_row (GtkWidget *file_list, GdkEvent *event, WPanel *panel)
 					      event->button.x, event->button.y,
 					      &row, &column);
 		gtk_clist_select_row (GTK_CLIST (file_list), row, 0);
-		gpopup_do_popup ((GdkEventButton *) event, panel, NULL, row, panel->dir.list[row].fname);
+		gpopup_do_popup ((GdkEventButton *) event, panel, NULL,
+				 row, panel->dir.list[row].fname);
 	}
 	return TRUE;
 }
@@ -1161,21 +1144,13 @@ panel_icon_list_scrolling_is_desirable (WPanel *panel, int x, int y)
 {
 	GtkAdjustment *va;
 
-#ifdef NEW_GNOME_LIBS
 	va = scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (panel->icons));
-#else
-	va = GNOME_ICON_LIST (panel->icons)->adj;
-#endif
 
 	if (y < 10) {
 		if (va->value > va->lower)
 			return TRUE;
 	} else {
-#ifdef NEW_GNOME_LIBS
 		if (y > (GTK_WIDGET (ILIST_FROM_SW (panel->icons))->allocation.height - 10)) {
-#else
-		if (y > (GTK_WIDGET (panel->icons)->allocation.height - 10)) {
-#endif
 			if (va->value < va->upper - va->page_size)
 				return TRUE;
 		}
@@ -1197,11 +1172,7 @@ panel_icon_list_scroll (gpointer data)
 	GtkAdjustment *va;
 	double v;
 
-#ifdef NEW_GNOME_LIBS
 	va = scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (panel->icons));
-#else
-	va = GNOME_ICON_LIST (panel->icons)->adj;
-#endif
 
 	if (panel->drag_motion_y < 10) {
 		v = va->value - va->step_increment;
@@ -1391,7 +1362,8 @@ panel_icon_list_select_icon (GtkWidget *widget, int index, GdkEvent *event, WPan
 	switch (event->type){
 	case GDK_BUTTON_PRESS:
 		if (event->button.button == 3)
-			gpopup_do_popup2 ((GdkEventButton *) event, panel, NULL, TRUE);
+			gpopup_do_popup ((GdkEventButton *) event, panel, NULL,
+					 index, panel->dir.list[index].fname);
 		break;
 
 	case GDK_BUTTON_RELEASE:
@@ -1459,10 +1431,13 @@ panel_icon_list_button_press (GtkWidget *widget, GdkEventButton *event, WPanel *
 
 	if (icon == -1) {
 		if (event->button == 3) {
-			gpopup_do_popup2 ((GdkEventButton *) event, panel, NULL, FALSE);
+#if 0
+			g_warning ("FIXME: icon_list_button_press menu");
+			gpopup_do_popup ((GdkEventButton *) event, panel, NULL, FALSE);
+#endif
 			return TRUE;
 		}
-	} else if (event->button == 1)
+	} else if (event->button != 3)
 		panel->maybe_start_drag = event->button;
 
 	panel->click_x = event->x;
@@ -1481,25 +1456,19 @@ panel_icon_list_button_release (GtkWidget *widget, GdkEventButton *event, WPanel
 static GtkWidget *
 panel_create_icon_display (WPanel *panel)
 {
-#ifdef NEW_GNOME_LIBS
 	GtkWidget *sw;
-#endif
 	GnomeIconList *ilist;
 
-#ifdef NEW_GNOME_LIBS
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#endif
 
 	ilist = GNOME_ICON_LIST (
 		gnome_icon_list_new_flags (90, NULL,
 					   (GNOME_ICON_LIST_IS_EDITABLE
 					    | GNOME_ICON_LIST_STATIC_TEXT)));
-#ifdef NEW_GNOME_LIBS
 	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (ilist));
 	gtk_widget_show (GTK_WIDGET (ilist));
-#endif
 
 	gnome_icon_list_set_separators (ilist, " /-_.");
 	gnome_icon_list_set_row_spacing (ilist, 2);
@@ -1563,11 +1532,7 @@ panel_create_icon_display (WPanel *panel)
 	gtk_signal_connect (GTK_OBJECT (ilist), "drag_leave",
 			    GTK_SIGNAL_FUNC (panel_icon_list_drag_leave), panel);
 
-#ifdef NEW_GNOME_LIBS
 	return sw;
-#else
-	return GTK_WIDGET (ilist);
-#endif
 }
 
 static void
@@ -2390,10 +2355,6 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	GtkWidget *status_line, *filter, *vbox, *ministatus_box;
 	GtkWidget *cwd;
 	GtkWidget *dock;
-#ifdef NEW_GNOME_LIBS
-#else
-	GtkWidget *box;
-#endif
 	GnomeUIBuilderData uibdata;
 
 	panel->xwindow = gtk_widget_get_toplevel (GTK_WIDGET (panel->widget.wdata));
@@ -2420,40 +2381,21 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	
 	panel->icons = panel_create_icon_display (panel);
 	gtk_widget_show (panel->icons);
-#ifdef NEW_GNOME_LIBS
-#else
-	panel->scrollbar = gtk_vscrollbar_new (GNOME_ICON_LIST (panel->icons)->adj);
-	gtk_widget_show (panel->scrollbar);
-#endif
-	
+
 	panel->list  = panel_create_file_list (panel);
 	gtk_widget_ref (panel->icons);
 	gtk_widget_ref (panel->list);
 
-#ifdef NEW_GNOME_LIBS
-#else
-	box = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), panel->icons, TRUE, TRUE, 0);
-	gtk_box_pack_end (GTK_BOX (box), panel->scrollbar, FALSE, TRUE, 0);
-	gtk_widget_show (box);
-#endif
-
 	panel->panel_listbox = gtk_event_box_new ();
 	gtk_widget_show (panel->panel_listbox);
 	gtk_container_add (GTK_CONTAINER (panel->panel_listbox), panel->list);
 
-#ifdef NEW_GNOME_LIBS
 	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), panel->icons, NULL);
-#else
-	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), box, NULL);
-#endif
 	gtk_notebook_append_page (GTK_NOTEBOOK (panel->notebook), panel->panel_listbox, NULL);
-	gtk_notebook_set_page (GTK_NOTEBOOK (panel->notebook), panel->list_type == list_icons ? 0 : 1);
-#ifdef NEW_GNOME_LIBS
+	gtk_notebook_set_page (GTK_NOTEBOOK (panel->notebook),
+			       panel->list_type == list_icons ? 0 : 1);
+
 	gtk_widget_show (panel->icons);
-#else
-	gtk_widget_show_all (box);
-#endif
 	gtk_widget_show (panel->list);
 	gtk_widget_show (panel->notebook);
 	
